@@ -5,6 +5,7 @@
 #include <QClipboard>
 #include <QMessageBox>
 #include <QFileDialog>
+#include <QTranslator>
 
 GenerateMenu::GenerateMenu(MainWindow *mainwindow, Operations *operations):QObject(mainwindow)
 {
@@ -33,7 +34,7 @@ QString GenerateMenu::find_icon(QString file_path, QString menu)
 	QDir dir(file_path);
 	if(!dir.exists())
 	{
-		printf("Error: %s not found\n", path.toLocal8Bit().data() );
+		printf("[GenerateMenu::find_icon] Error: %s not found\n", path.toLocal8Bit().data() );
 		return QString();
 	}
 	QFileInfoList list=dir.entryInfoList( QDir::NoFilter, QDir::Name );
@@ -58,12 +59,49 @@ void GenerateMenu::load_menu()
 void GenerateMenu::load_menu(QString dir_name, QMenu *parent_menu)
 {
 	QDir dir(dir_name);
+	QTranslator *qTranslator; // For all translation files founded
+
 	if(!dir.exists())
 	{
-		printf("Error: %s not found\n", path.toLocal8Bit().data());
+		printf("[GenerateMenu::load_menu] Error: %s not found\n", path.toLocal8Bit().data());
 		return;
 	}
-	QFileInfoList list=dir.entryInfoList( QDir::NoFilter, QDir::Name );
+
+	// Find translations
+	QStringList loadedTranslations;
+	QStringList filter;
+	filter << "*.qm";
+	QFileInfoList list = dir.entryInfoList(filter, QDir::Files | QDir::Readable);
+					      
+	for(int i = 0; i < list.size(); ++i)
+	{
+	  QFileInfo fileInfo;
+	  QString   tName;
+
+	  fileInfo    = list.at(i);
+	  tName       = fileInfo.fileName().split("_").first();
+
+	  if(loadedTranslations.indexOf(tName) == -1)
+	  {
+	    qTranslator = new QTranslator(this);
+	    loadedTranslations.append(tName);
+	  
+	    if(!qTranslator->load(tName + "_" + QLocale::system().name(), fileInfo.absolutePath()))
+	    {
+	      delete qTranslator;
+	      printf("[GenerateMenu::load_menu] Unknow error loading translation file '%s'\n",
+		     tName.toLocal8Bit().data());
+	    }else
+	    {
+	      QCoreApplication::instance()->installTranslator(qTranslator);
+	      printf("[GenerateMenu::load_menu] Translation file founded: '%s'\n",
+		     tName.toLocal8Bit().data());
+	    }
+	  }
+	}
+
+	// Process
+	list = dir.entryInfoList( QDir::NoFilter, QDir::Name );
 	for (int i = 0; i < list.size(); ++i) {
 		QFileInfo fileInfo = list.at(i);
 		QString menu_name=fileInfo.fileName();
@@ -72,14 +110,19 @@ void GenerateMenu::load_menu(QString dir_name, QMenu *parent_menu)
 		{
 			//printf("Icono encontrado %s\n",find_icon(dir_name,menu_name).toLocal8Bit().data());
 			//printf("Menu path %s\n",menu_path(dir_name+"/"+menu_name).toLocal8Bit().data());
-			QMenu *menu=mainwindow->createMenu(menu_path(dir_name+"/"+menu_name),menu_name, find_icon(dir_name,menu_name));
+			QMenu *menu=mainwindow->createMenu(menu_path(dir_name+"/"+menu_name),
+							   MENU_TRANSLATE(menu_name),
+							   find_icon(dir_name,menu_name));
 			if(parent_menu==NULL) mainwindow->menubar->addAction(menu->menuAction());
 			else parent_menu->addAction(menu->menuAction());
 			load_menu(dir_name+"/"+menu_name,menu);
 		}
 		else if(fileInfo.isExecutable() && !menu_name.startsWith("."))
 		{
-			QAction *action = mainwindow->createAction(menu_path(dir_name+"/"+menu_name),menu_name, find_icon(dir_name,menu_name));
+			QAction *action = mainwindow->createAction(menu_path(dir_name+"/"+menu_name),
+								   MENU_TRANSLATE(menu_name),
+								   find_icon(dir_name,menu_name));
+
 			mainwindow->menus[menu_path(dir_name)]->addAction(action);
 			MenuCallBack *callback=new MenuCallBack(dir_name+"/"+menu_name, mainwindow->octave_connection);
 			connect(action, SIGNAL(triggered()), callback, SLOT(actions_callback()));
@@ -107,7 +150,12 @@ void GenerateMenu::load_menu(QString dir_name, QMenu *parent_menu)
 			
 			QString _menu_name=wizard-> windowTitle();
 			
-			QAction *action = mainwindow->createAction( menu_path(dir_name+"/"+_menu_name), _menu_name,  find_icon(dir_name,menu_name) );
+			QAction *action = mainwindow->createAction( menu_path(dir_name+"/"+_menu_name),
+								    MENU_TRANSLATE(_menu_name),
+								    find_icon(dir_name, menu_name) );
+
+			printf("%s / %s\n", _menu_name.toLocal8Bit().data(),
+			       QCoreApplication::translate("", "Load matrix from file").toLocal8Bit().data());
 			
 			mainwindow->menus[menu_path(dir_name)]->addAction(action);
 			
